// $Id: hierarchical_select.js,v 1.15 2007/12/30 18:42:18 wimleers Exp $

var HierarchicalSelect = HierarchicalSelect || {};

HierarchicalSelect.dropboxContent = new Array();

HierarchicalSelect.context = function() {
  return $("form div.form-item");
};

HierarchicalSelect.setting = function(hsid, settingName, newValue) {
  // Global settings.
  if (hsid == 'global') {
    return Drupal.settings.hierarchical_select[settingName];
  }
  else {
    // Per-Hierarchical Select settings.
    if (undefined === newValue) {
      return Drupal.settings.hierarchical_select.settings[hsid][settingName];
    }
    else {
      Drupal.settings.hierarchical_select.settings[hsid][settingName] = newValue;
    }
  }
};

HierarchicalSelect.updateOriginalSelect = function(hsid) {
  var $selects = $('select.hierarchical-select-'+ hsid +'-select', this.context);

  // Reset the current selection in the original select.
  $('select.hierarchical-select-'+ hsid  +'-original-select option:selected', this.context).each(function() {
    $(this).removeAttr('selected');
  });

  var rootLevelValue = $('select#hierarchical-select-'+ hsid +'-select-level-0', this.context).val();
  // Update it to the current selection.
  if (typeof(rootLevelValue) == "string" && rootLevelValue.match(/^(none|label_\d+)$/)) {
    // This is for compatibility with Drupal's Taxonomy form items. They have
    // a "- None selected -" option, with the value "". We *must* select it if
    // we want to select nothing.
    // TODO: make sure Drupal standardizes on a form item with a value "" to
    // select nothing. Perhaps I should also make Hierarchical Select use this
    // for its "<none>" option?
    $('select.hierarchical-select-'+ hsid +'-original-select', this.context)
    .find('option[@value=""]')
    .attr('selected', 'selected'); 
  }
  else if (!this.setting(hsid, 'saveLineage')) {
    // Get the deepest valid value of the current selection.
    var level = $selects.length - 1;
    var deepestSelectValue = '';
    do {
      deepestSelectValue = $selects.eq(level).val();
      level--;
    } while (level >= 0 && deepestSelectValue.match(/label_\d+/));

    // Update the original select.
    $('select.hierarchical-select-'+ hsid +'-original-select', this.context)
    .val(deepestSelectValue);
  }
  else {
    // Select each hierarchical select's selected option in the original
    // select (thus effectively saving the term lineage).
    $selects
    .each(function() { // Can be done cleaner in jQuery 1.2, because .val() can then accept an array.
      $('select.hierarchical-select-'+ hsid +'-original-select', this.context)
      .find('option[@value='+ $(this).val() +']')
      .attr('selected', 'selected');
    });
  }

  // If multiple select is enabled, also add the selections in the dropbox.
  if (this.setting(hsid, 'multiple')) {
    for (var i = 0; i < HierarchicalSelect.dropboxContent[hsid].length; i++) {
      for (var j = 0; j < HierarchicalSelect.dropboxContent[hsid][i].length; j++) {
        $('select.hierarchical-select-'+ hsid +'-original-select', this.context)
        .find('option[@value='+ HierarchicalSelect.dropboxContent[hsid][i][j] +']')
        .attr('selected', 'selected');
      }
    }
  }
};

HierarchicalSelect.initialize = function() {
  for (var hsid in Drupal.settings.hierarchical_select.settings) {
    // If multiple select is enabled, intialize the dropbox content array.
    if (this.setting(hsid, 'multiple')) {
      HierarchicalSelect.dropboxContent[hsid] = this.setting(hsid, 'initialDropboxLineagesSelections');
    }

    $('select.hierarchical-select-'+ hsid +'-original-select', this.context)
    // Hide the standard select.
    .hide(0)
    // Add a unique container div after the standard select.
    .after('<div id="hierarchical-select-'+ hsid +'-container" class="hierarchical-select-container clear-block" />');

    // Now load the initial HTML *without* using AHAH.
    $('div#hierarchical-select-'+ hsid +'-container', this.context)
    .html(this.setting(hsid, 'initial'));

    this.updateOriginalSelect(hsid);
    this.attachBindings(hsid);
  }
};

HierarchicalSelect.attachBindings = function(hsid, dropboxOnly) {
  // Update event: attach to every select of the current Hierarchical Select.
  $('select.hierarchical-select-'+ hsid +'-select', this.context)
  .unbind()
  .change(function(x) {
    return function() { HierarchicalSelect.update(x, $(this).val()); };
  }(hsid));

  if (this.setting(hsid, 'multiple')) {
    if (!dropboxOnly) {
      // Add the "Add" button.
      $('div#hierarchical-select-'+ hsid +'-container .hierarchical-select-input', this.context)
      .append(this.setting(hsid, 'addButton'));

      // Add event: attach to the "Add" button.
      $('#hierarchical-select-'+ hsid +'-add-to-dropbox', this.context)
      .unbind()
      .click(function(x) {
        return function() { HierarchicalSelect.add(x); };
      }(hsid));
    }

    for (var i = 0; i < this.dropboxContent[hsid].length; i++) {
      // Remove event: attach to the "Remove" links.
      $('#hierarchical-select-'+ hsid +'-remove-'+ i + '-from-dropbox', this.context)
      .unbind()
      .click(function(x, y) {
        return function() { HierarchicalSelect.remove(x, y); };
      }(hsid, i));
    }    
  }
};

HierarchicalSelect.getFullSelection = function(hsid, selection) {
  var $selects = $('select.hierarchical-select-'+ hsid +'-select', this.context);

  // Make sure selection is always an array.
  if ("string" == typeof(selection)) {
    selection = new Array(selection);
  }

  if (this.setting(hsid, 'saveLineage')) {
    var lineageSelection = new Array();
    for (var level = 0; level < $selects.size(); level++) {
      var s = $('select#hierarchical-select-'+ hsid +'-select-level-'+ level, this.context).val();
      lineageSelection[level] = s;
      if (s == selection) {
        // Don't go collect values from levels deeper than the clicked level,
        // they have to be tossed away anyway.
        break;
      }
    }
  }

  return (undefined === lineageSelection) ? selection : lineageSelection;
};

HierarchicalSelect.post = function(hsid, fullSelection, type) {
  var post = new Object();
  post['hsid'] = hsid;
  if (typeof(fullSelection) == "string") {
    post['selection'] = fullSelection;
  }
  else {
    post['selection'] = fullSelection.join('|');
  }
  post['module'] = this.setting(hsid, 'module');
  post['save_lineage'] = this.setting(hsid, 'saveLineage');
  post['enforce_deepest'] = this.setting(hsid, 'enforceDeepest');
  post['level_labels'] = this.setting(hsid, 'levelLabels');
  post['params'] = this.setting(hsid, 'params');
  post['required'] = this.setting(hsid, 'required');
  post['type'] = type;

  return post;
};

HierarchicalSelect.add = function(hsid) {
  var HS = HierarchicalSelect;
  var $selects = $('select.hierarchical-select-'+ hsid +'-select', HS.context);

  // Get all selected items.
  var fullSelection = new Array();
  $('select.hierarchical-select-'+ hsid  +'-original-select option:selected', this.context).each(function() {
    fullSelection.push($(this).val());
  });

  $.ajax({
    type: "POST",
    url: HS.setting('global', 'url'),
    data: HS.post(hsid, fullSelection, 'dropbox'),
    dataType: "json",
    success: function(json){
      $('div#hierarchical-select-'+ hsid +'-container .hierarchical-select-input', HS.context)
      .html(json.selects);
      $('div#hierarchical-select-'+ hsid +'-container .hierarchical-select-dropbox', HS.context)
      .html(json.dropbox);

      HS.dropboxContent[hsid] = json.dropboxLineagesSelections;

      HS.updateOriginalSelect(hsid); // In theory we don't have to do this, but it's a safety net: it will only set valid selections.
      HS.attachBindings(hsid);
    }
  });
};

HierarchicalSelect.remove = function(hsid, dropboxEntry) {
  var HS = HierarchicalSelect;
  var $selects = $('select.hierarchical-select-'+ hsid +'-select', HS.context);

  // Add the selections of all items in the dropbox to the selection, except
  // for the one that has to be removed. If we submit this, the server will
  // reconstruct all lineages and thus remove the removed selection.
  var fullSelection = new Array();
  for (var i = 0; i < HierarchicalSelect.dropboxContent[hsid].length; i++) {
    if (i != dropboxEntry) {
      for (var j = 0; j < HierarchicalSelect.dropboxContent[hsid][i].length; j++) {
        fullSelection.push(HierarchicalSelect.dropboxContent[hsid][i][j]);
      }
    }
  }

  $.ajax({
    type: "POST",
    url: HS.setting('global', 'url'),
    data: HS.post(hsid, fullSelection, 'dropbox-remove'),
    dataType: "json",
    success: function(json){
      $('div#hierarchical-select-'+ hsid +'-container .hierarchical-select-dropbox', this.context)
      .html(json.dropbox);

      HS.dropboxContent[hsid] = json.dropboxLineagesSelections;

      HS.updateOriginalSelect(hsid);
      HS.attachBindings(hsid, true);
    }
  });
};

HierarchicalSelect.update = function(hsid, selection) {
  var HS = HierarchicalSelect;

  var animationDelay = HS.setting(hsid, 'animationDelay');

  var $selects = $('select.hierarchical-select-'+ hsid +'-select', HS.context);
  var lastUnchanged = $selects.index($('select.hierarchical-select-'+ hsid +'-select option[@value='+ selection +']', HS.context).parent()[0]);

  // Drop out the *original* selects of the levels deeper than the select of
  // the level that just changed.
  $selects.gt(lastUnchanged).DropOutLeft(animationDelay);

  $.ajax({
    type: "POST",
    url: HierarchicalSelect.setting('global', 'url'),
    data: HierarchicalSelect.post(hsid, HS.getFullSelection(hsid, selection), 'selects'),
    dataType: "json",
    success: function(json){
      $('div#hierarchical-select-'+ hsid +'-container .hierarchical-select-input', HS.context)
      .html(json.html);

      $selects = $('select.hierarchical-select-'+ hsid + '-select', HS.context);

      // Hide the loaded selects after the one that was just changed, then  drop
      // them in.
      $selects.gt(lastUnchanged).hide(0).DropInLeft(animationDelay);

      HS.updateOriginalSelect(hsid);
      HS.attachBindings(hsid);
    }
  });
};

if (Drupal.jsEnabled) {
  $(document).ready(function() {
    HierarchicalSelect.initialize();
  });
}
